   <?php
   /*
   * 2007-2015 PrestaShop
   *
   * NOTICE OF LICENSE
   *
   * This source file is subject to the Academic Free License (AFL 3.0)
   * that is bundled with this package in the file LICENSE.txt.
   * It is also available through the world-wide-web at this URL:
   * http://opensource.org/licenses/afl-3.0.php
   * If you did not receive a copy of the license and are unable to
   * obtain it through the world-wide-web, please send an email
   * to license@prestashop.com so we can send you a copy immediately.
   *
   * DISCLAIMER
   *
   * Do not edit or add to this file if you wish to upgrade PrestaShop to newer
   * versions in the future. If you wish to customize PrestaShop for your
   * needs please refer to http://www.prestashop.com for more information.
   *
   *  @author PrestaShop SA <contact@prestashop.com>
   *  @copyright  2007-2015 PrestaShop SA
   *  @license    http://opensource.org/licenses/afl-3.0.php  Academic Free License (AFL 3.0)
   *  International Registered Trademark & Property of PrestaShop SA
   */

   include dirname(__FILE__)."/SL/SL_Invoice.php";

   use PrestaShop\PrestaShop\Core\Payment\PaymentOption;

   if (!defined('_PS_VERSION_')) {exit;}
   class Smartlink extends PaymentModule {
      const FLAG_DISPLAY_PAYMENT_INVITE = 'SMARTLINK_PAYMENT_INVITE';

      public function __construct() {
         $this->name = 'smartlink';
         $this->tab = 'wallets';
         $this->version = '1.0.2';
         $this->ps_versions_compliancy = array('min' => '1.7', 'max' => _PS_VERSION_);
         $this->author = 'Smartlink';
         $this->need_instance = 1;
         $this->is_eu_compatible = 1;
         $this->currencies = true;
         $this->currencies_mode = 'checkbox';
         $this->bootstrap = true;
         $this->controllers = array('redirect', 'callback', 'webhook');
         parent::__construct();

         $this->tabParentName = 'AdminTools';
         $this->displayName = $this->l('Smartlink Crypto Payment Gateway');
         $this->description = $this->l("Welcome to the Smartlink WooCommerce Plugin Setup! Setting up your Smartlink payment gateway is a breeze, but first, let's ensure you have your Smartlink Merchant ID and API Key handy. If you do not have these details yet, you can easily obtain them by logging into your Smartlink merchant account.");
         $this->confirmUninstall = $this->l('Are you sure you want to delete your details?');
         $this->tabClassName = 'SmartlinkWallets';
      }

      public function install(){ // Create smartlink config table on plugin install
          
         if (!parent::install() || !$this->registerHook('paymentOptions') || !$this->registerHook('paymentReturn')) {
            return false;
         }
         
         $this->registerHook('displayHeader');
         $this->registerHook('displayAdminOrder');
         $this->registerHook('displayOrderDetail');
         $this->registerHook('moduleRoutes');

         $table_name = 'smartlink_payments';
         $table_name2 = 'smartlink_webhook';

         $sql = "CREATE TABLE IF NOT EXISTS $table_name(
            `id` int(11) NOT NULL AUTO_INCREMENT,
            `order_id` int(11) NOT NULL,
            `user_id` int(11) NOT NULL,
            `crypto_amount` decimal(20,8) NOT NULL,
            `fiat_amount` decimal(20,2) NOT NULL,
            `fiat_currency` varchar(255) NOT NULL,
            `crypto_currency` varchar(255) NOT NULL,
            `trx_id`text NOT NULL,
            `status` tinyint(1) NOT NULL,
            `created_at` datetime NOT NULL DEFAULT CURRENT_TIMESTAMP,
            `updated_at` datetime NOT NULL DEFAULT CURRENT_TIMESTAMP,
            PRIMARY KEY (`id`)
         )";

         $sql2 = "CREATE TABLE IF NOT EXISTS $table_name2(
            `id` int(11) NOT NULL AUTO_INCREMENT,
            `order_id` int(11) NOT NULL,
            `trx_id` text NOT NULL,
            `type` int(11) NOT NULL,
            `explorer_url` varchar(255) NOT NULL,
            `paid_amount` decimal(20,8) NOT NULL,
            `paid_currency` varchar(255) NOT NULL,
            `paid_date` datetime NOT NULL DEFAULT CURRENT_TIMESTAMP,
            `confirmation` int(11) NOT NULL,
            `created_at` datetime NOT NULL DEFAULT CURRENT_TIMESTAMP,
            `updated_at` datetime NOT NULL DEFAULT CURRENT_TIMESTAMP,
            PRIMARY KEY (`id`)
         )";

         $db = Db::getInstance();
         $db->Execute($sql);
         $db->Execute($sql2);

         Configuration::updateValue('smartlink_marchant_id', '');
         Configuration::updateValue('smartlink_title', 'Pay Using Smartlink');
         Configuration::updateValue('smartlink_api_key', '');
         $id_tab = Tab::getIdFromClassName($this->tabClassName);
         if (!$id_tab) {
            $tab = new Tab();
            $tab->class_name = $this->tabClassName;
            $tab->id_parent = Tab::getIdFromClassName($this->tabParentName);
            $tab->module = $this->name;
            $tab->active=1;
            $languages = Language::getLanguages();
            foreach ($languages as $language)
            $tab->name[$language['id_lang']] = $this->displayName;
            $tab->add();
         }

         return true;
      }
      
      protected function getConfigForm(){ // Admin panel smartlink plugin configuration form
         return array(
            'form' => array(
               'legend' => array(
                  'title' => $this->l('Settings'),
                  'icon' => 'icon-cogs',
               ),
               'input' => array(
                  
                  array(
                        'type' => 'text',
                        'label' => $this->l('Marchant Id'),
                        'name' => 'smartlink_marchant_id',
                        'value' => '',
                        'desc' => $this->l('This is your unique identifier within the Smartlink ecosystem. It allows Smartlink to recognize your business and link transactions to your merchant account.'),
                  ),
                  array(
                        'type'  => 'text',
                        'label' => $this->l('Api Key'),
                        'name'  => 'smartlink_api_key',
                        'value' => '',
                        'desc'  => $this->l('This serves as the secret password between your WooCommerce store and Smartlink. It allows for secure communication of transaction details.'),
                  ),
                  array(
                        'type'  => 'text',
                        'label' => $this->l('Title'),
                        'name'  => 'smartlink_title',
                        'value' => 'Pay Using Smartlink',
                        'desc'  => $this->l("This is the name that will appear as your payment method during checkout. Make sure it's recognizable and aligns with your brand."),
                  ),
               ),

               'submit' => array(
                  'title' => $this->l('Save'),
               ),
            ),
         );
      }
      
      public function getContent(){ 

         $output = null;
         if (Tools::isSubmit('submit_smartlink')) {
            $output_array = $this->postProcess();
            if($output_array['err'] == 0){
               $output .= $this->displayConfirmation($this->l($output_array['msg']));
            }else{
               $output .= $this->displayError($this->l($output_array['msg']));
            }
         }

         $this->context->smarty->assign(array('module_dir' => $this->_path));

         return $output.$this->renderForm();
      }
      
      protected function getConfigFormValues(){ // get general plugin configuration

         return array(
            'smartlink_api_key' => Configuration::get('smartlink_api_key', true),
            'smartlink_marchant_id' => Configuration::get('smartlink_marchant_id', true),
            'smartlink_title' => Configuration::get('smartlink_title', true),
         );
      }

      protected function renderForm(){ // render forms
         $helper = new HelperForm();

         $helper->show_toolbar = false;
         $helper->table = $this->table;
         $helper->module = $this;
         $helper->default_form_language = $this->context->language->id;
         $helper->allow_employee_form_lang = Configuration::get('PS_BO_ALLOW_EMPLOYEE_FORM_LANG', 0);

         $helper->identifier = $this->identifier;
         $helper->submit_action = 'submit_smartlink';
         $helper->currentIndex = $this->context->link->getAdminLink('AdminModules', false)
         . '&configure=' . $this->name . '&tab_module=' . $this->tab . '&module_name=' . $this->name;
         $helper->token = Tools::getAdminTokenLite('AdminModules');

         $helper->tpl_vars = array(
            'fields_value' => $this->getConfigFormValues(), /* Add values for your inputs */
            'languages' => $this->context->controller->getLanguages(),
            'id_language' => $this->context->language->id,
         );

         return $helper->generateForm(array($this->getConfigForm()));
      }

      protected function generateForm(){
         $invoice = new SL_Invoice();
         $cart = $this->context->cart;
         $currency = $this->context->currency;
         $total = (string)$cart->getOrderTotal(true, Cart::BOTH);

         $add_param['api_key'] = Configuration::get('smartlink_api_key', true);
         $add_param['merchant_id'] = Configuration::get('smartlink_marchant_id', true);
         $add_param['fiat_amount'] = $total;
         $add_param['fiat_currency'] = $currency->iso_code;
         
         $order_data = $invoice->Sl_validateOrder($add_param);
         if(isset($order_data['flag']) && $order_data['flag'] == 0){
            $this->context->smarty->assign(array(
               'message' => $order_data['msg'],
            ));
            return $this->context->smarty->fetch($this->local_path.'views/templates/front/payment_form.tpl');
         }
         if(isset($order_data['flag']) && $order_data['flag'] == 2){
            $this->context->smarty->assign(array(
               'message' => $order_data['msg'],
            ));
            return $this->context->smarty->fetch($this->local_path.'views/templates/front/payment_form.tpl');
         }else{
            $this->context->smarty->assign(array(
            'action' => $this->context->link->getModuleLink($this->name, 'redirect', array(), true),
            'message' => 'You are paying with smartlink',
         ));
         return $this->context->smarty->fetch($this->local_path.'views/templates/front/payment_form.tpl');
         }
      }

      public function hookModuleRoutes()
      {
         return [
            'module-smartlink-ipn' => [
               'rule' => 'smartlink/callback',
               'keywords' => [],
               'controller' => 'callback',
               'params' => [
                  'fc' => 'module',
                  'module' => 'smartlink',
               ],
               'rule' => 'smartlink/webhook',
               'keywords' => [],
               'controller' => 'webhook',
               'params' => [
                  'fc' => 'module',
                  'module' => 'smartlink',
               ],
            ],
         ];
      }

      // public function viewAccess($disable = false){
      //    $result = true;
      //    return $result;
      // }


      public function postProcess(){  // Validate merchant details

         
         //make validation
         $form_values = $this->getConfigFormValues();
         foreach (array_keys($form_values) as $key) {
               $config_value = Tools::getValue($key);
               Configuration::updateValue($key, $config_value);
            }
            $invoice = new SL_Invoice();
            $param['api_key'] = $this->getConfigFormValues()['smartlink_api_key'];
            $param['merchant_id'] = $this->getConfigFormValues()['smartlink_marchant_id'];
            $param['webhook_url'] = $this->context->link->getModuleLink('smartlink','webhook');
            $validate = $invoice->SL_validateMerchant($param);
            if($validate['flag'] == 1){
               $return_array = array('err' => 0, 'msg' =>'Configuration updated successfully');
               Configuration::updateValue("smartlink_web_id", $validate['data']['web_id']);
               Configuration::updateValue("smartlink_merchant_email", $validate['data']['email']);
               Configuration::updateValue("smartlink_merchant_name", $validate['data']['merchant_name']);
               Configuration::updateValue("smartlink_whitelist_domain", $validate['data']['whitelist_domain']);
               Configuration::updateValue("smartlink_merchant_image", $validate['data']['merchant_image']);
               Configuration::updateValue("smartlink_fiat_currency", $validate['data']['fiat_currency']);
               Configuration::updateValue("smartlink_platform", $validate['data']['platform']);
               Configuration::updateValue("smartlink_redirect_url", $validate['data']['redirect_url']);
            }else{
               $return_array = array('err' => 1, 'msg' =>'Inavlide API Key or Merchant ID');
            }
         return $return_array;
         
      }

      public function hookPaymentOptions()
      {
         if (!$this->active || !empty($this->warning)) {
            return;
         }

         $paymentOption = new PaymentOption();
         $paymentOption->setCallToActionText(Configuration::get('smartlink_title'))->setForm($this->generateForm());
         $payment_options = array($paymentOption);

         return $payment_options;
      }

      public function hookDisplayOrderDetail($order){
         $order = json_decode(json_encode($order),true);
         $orderid = Tools::getValue('id_order');
         $check_pay = $this->getPaymentDetail($orderid);
         if($order['order']['payment'] == 'Smartlink' && !empty($check_pay)){
            return $this->front_order_page_detail($orderid);
         }
         return '';
      }

      public function getPaymentDetail($orderid){ // get payment detail by order id
         $results =[];
         $wtable = 'smartlink_payments';
         $bp_sql = "SELECT * FROM $wtable WHERE order_id = '$orderid' LIMIT 1";
         $results = Db::getInstance()->executes($bp_sql);
         if(count($results) > 0){
            $results =  $results[0];
         }
         return $results;
      }

      public function front_order_page_detail($orderid){ // frontend order detail page
         // $this->updateOrderStatusInDetail($orderid);
         $payment_detail= $this->getPaymentDetail($orderid);
         
         $url = "";
         $invoice_id = "";
         $address = "";
         $status = "";
         $date = "";
         $currency = "";
         $paid_amt = 0;
         $pending_amount  = 0;
         $total_amount = 0;
         $payment_history = [];

         $sql = "SELECT * FROM smartlink_payments WHERE order_id = '$orderid' LIMIT 1";
         $results = Db::getInstance()->executes($sql);
         $data = $results[0];
         $currency = $data['crypto_currency'];
         $date = $data['created_at'];
         $status = $data['status'] == 0 ? 'Pending' : 'Paid' ;
         $total_amount = $data['crypto_amount'];

         $sql= "SELECT * FROM smartlink_webhook WHERE order_id = '$orderid'";
         $results = Db::getInstance()->executes($sql);
         foreach ($results as $row) {
            if($row['confirmation'] == 1)
               $paid_amt += $row['paid_amount'];
            $r = array();
            $r['txid'] = $row['trx_id'];
            $r['explorer_url'] = $row['explorer_url'];
            array_push($payment_history, $r);
         }

         $pending_amount = $total_amount - $paid_amt;
         if($pending_amount < 0){
            $pending_amount = '0.00000000';
         }
         $this->context->smarty->assign(array(
            'url' => $url,
            'invoice_id' => $invoice_id,
            'address' => $address,
            'status' => $status,
            'date' => $date,
            'currency' => $currency,
            'pending_amount' => number_format($pending_amount,8,".",""),
            'paid_amount' => number_format($paid_amt,8,".",""),
            'total_amount' => number_format($total_amount,8,".",""),
            'payment_history' => $payment_history,
            'invoice_url' => _PS_BASE_URL_ . __PS_BASE_URI__ .'index.php?controller=invoice&fc=module&module=smartlink&order_id='.$orderid
         ));
         return $this->context->smarty->fetch($this->local_path.'views/templates/hook/front_order_detail.tpl');
      }
   
      public function uninstall(){ // code run on uninstall plugin
         $id_tab = Tab::getIdFromClassName($this->tabClassName);
         if ($id_tab) {
            $tab = new Tab($id_tab);
            $tab->delete();
         }
         return parent::uninstall();
      }
   }
